<?php

namespace App\Console\Commands;

use Exception;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use PHPMailer\PHPMailer\PHPMailer;
use Webklex\PHPIMAP\ClientManager;

class FilterAndForwardEmails extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'emails:filter-and-forward';



    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Filters and forwards emails based on specific criteria';

    /**
     * Execute the console command.
     */
    public function handle()
    {
        //
        $accounts = config('emails.accounts');
        $forwardTo = config('emails.forward_to');

        foreach ($accounts as $account) {
            $this->filterEmails($account, $forwardTo);
        }

        $this->info('Filtered and forwarded emails for all accounts successfully.');
    }
    private function filterEmails($account, $forwardTo)
    {
        $client = (new ClientManager())->make([
            'host'          => 'rcons.org',
            'port'          => 993,
            'encryption'    => 'ssl',
            'validate_cert' => true,
            'username'      => $account['username'],
            'password'      => $account['password'],
            'protocol'      => 'imap',
            'authentication' => null,
        ]);

        try {
            $client->connect();

            $inbox = $client->getFolder('INBOX');
            $messages = $inbox->query()->unseen()->get(); // Get only unseen emails

            foreach ($messages as $message) {
                $fromAddress = $message->getFrom()[0]->mail; // Get the sender's email address
                $subject = $message->getSubject();
                $body = $message->getTextBody();

                // Add your custom filter logic here
                $isWarmUpEmail = strpos($body, 'ALi'); // Example identifier

                if (strpos($subject, 'ALi') !== false || strpos($body, 'Ali') !== false) {
                    // Forward the email if it passes your custom logic
                    $this->forwardEmail($forwardTo, $subject, $body, $fromAddress);

                    // Optionally, mark the message as seen
                    $message->setFlag('\\Seen');
                }
            }

            $client->disconnect();
        } catch (\Exception $e) {
            $this->error("Error connecting to the IMAP server: " . $e->getMessage());
        }
    }

    // private function forwardEmail($to, $subject, $body, $from)
    // {
    //     $mail = new PHPMailer(true);
    //     try {
    //         $mail->isSMTP();
    //         $mail->Host = 'rcons.org';
    //         $mail->SMTPAuth = true;
    //         $mail->Username = env('MAIL_USERNAME');
    //         $mail->Password = env('MAIL_PASSWORD');
    //         $mail->SMTPSecure = 'ssl';
    //         $mail->Port = 465;

    //         // Set who the email is to be sent from
    //         // Set from address to match the domain
    //         $mail->setFrom(env('MAIL_FROM_ADDRESS'), env('MAIL_FROM_NAME'));

    //         // Set the original sender's email in "Reply-To"
    //         $mail->addReplyTo($from, 'Original Sender');

    //         // Set who the email is to be sent to
    //         $mail->addAddress($to);

    //         // Subject and body
    //         $mail->Subject = $subject;
    //         $mail->Body    = $body;

    //         $mail->send();
    //         Log::info("Forwarded email from $from to $to with subject $subject.");
    //         $this->info("Forwarded email from $from to $to with subject $subject.");
    //     } catch (Exception $e) {
    //         Log::error("Message could not be sent. Mailer Error: {$mail->ErrorInfo}");
    //         $this->info("Message could not be sent. Mailer Error: {$mail->ErrorInfo}");
    //     }
    // }
    private function forwardEmail($to, $subject, $body, $from)
    {
        $mail = new PHPMailer(true);

        // Retrieve warmup identifiers from the config file
        $warmupIdentifiers = config('warmup.identifiers');

        // Check if any warmup identifier is found in the body
        $identifierFound = false;
        foreach ($warmupIdentifiers as $identifier) {
            if (strpos($body, $identifier) !== false || strpos($subject, $identifier) !== false) {
                $identifierFound = true;
                break;
            }
        }

        if ($identifierFound) {
            try {
                $mail->isSMTP();
                $mail->Host = 'rcons.org';
                $mail->SMTPAuth = true;
                $mail->Username = env('MAIL_USERNAME');
                $mail->Password = env('MAIL_PASSWORD');
                $mail->SMTPSecure = 'ssl';
                $mail->Port = 465;

                // Set from address to match the domain
                $mail->setFrom(env('MAIL_FROM_ADDRESS'), env('MAIL_FROM_NAME'));

                // Set the original sender's email in "Reply-To"
                $mail->addReplyTo($from, 'Original Sender');

                // Set who the email is to be sent to
                $mail->addAddress($to);

                // Subject and body
                $mail->Subject = $subject;
                $mail->Body    = $body;

                $mail->send();
                Log::info("Forwarded email from $from to $to with subject $subject containing a warmup identifier.");
                $this->info("Forwarded email from $from to $to with subject $subject containing a warmup identifier.");
            } catch (Exception $e) {
                Log::error("Message could not be sent. Mailer Error: {$mail->ErrorInfo}");
                $this->info("Message could not be sent. Mailer Error: {$mail->ErrorInfo}");
            }
        } else {
            Log::info("No warmup identifier found in the email body. Email not sent.");
            $this->info("No warmup identifier found in the email body. Email not sent.");
        }
    }
}
